@@ -1,4 +1,4 @@
-# Copyright (c) 2017 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2017-2018 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -13,8 +13,8 @@
 from gi.repository import Gtk, Gdk, GLib, Pango, GObject, WebKit2
 
 from eolie.label_indicator import LabelIndicator
-from eolie.define import El, ArtSize
-from eolie.utils import remove_www
+from eolie.define import App, ArtSize
+from eolie.logger import Logger
 
 
 class PageChildRow(Gtk.ListBoxRow):
@@ -60,6 +60,14 @@ class PageChildRow(Gtk.ListBoxRow):
         view.webview.connect("title-changed", self.__on_webview_title_changed)
         eventbox.connect("button-press-event", self.__on_button_press_event)
 
+    def destroy(self):
+        """
+            Disconnect and destroy
+        """
+        self.__view.webview.disconnect_by_func(self.__on_webview_load_changed)
+        self.__view.webview.disconnect_by_func(self.__on_webview_title_changed)
+        Gtk.ListBoxRow.destroy(self)
+
     @property
     def view(self):
         """
@@ -148,6 +156,7 @@ class SitesManagerChild(Gtk.ListBoxRow):
         self.__window = window
         self.__netloc = netloc
         self.__minimal = None
+        self.__current_child = None
         self.__ephemeral = ephemeral
         self.__views = []
         self.__connected_ids = []
@@ -169,7 +178,6 @@ class SitesManagerChild(Gtk.ListBoxRow):
         self.__netloc_label.set_text(self.__netloc)
         self.__image = builder.get_object("image")
         self.__image.set_property("pixel-size", ArtSize.FAVICON)
-        self.__set_artwork()
         self.add(widget)
         self.drag_source_set(Gdk.ModifierType.BUTTON1_MASK, [],
                              Gdk.DragAction.MOVE)
@@ -196,7 +204,9 @@ class SitesManagerChild(Gtk.ListBoxRow):
                                  self.__on_webview_favicon_changed)
             self.update_label()
             self.__indicator_label.update_count(True)
-            if not view.webview.shown:
+            if view.webview.shown:
+                self.__on_webview_favicon_changed(view.webview)
+            else:
                 self.__indicator_label.mark_unshown(view.webview)
             if self.__pages_listbox is not None:
                 child = PageChildRow(view, self.__window)
@@ -219,8 +229,10 @@ class SitesManagerChild(Gtk.ListBoxRow):
             if self.__pages_listbox is not None:
                 for child in self.__pages_listbox.get_children():
                     if child.view == view:
-                        self.__pages_listbox.remove(child)
+                        child.destroy()
                         break
+        if self.__views:
+            self.__on_webview_favicon_changed(self.__views[0].webview)
 
     def set_minimal(self, minimal):
         """
@@ -243,6 +255,7 @@ class SitesManagerChild(Gtk.ListBoxRow):
             self.__grid.remove(self.__indicator_label)
             self.__grid.attach(self.__netloc_label, 1, 0, 1, 1)
             self.__pages_listbox = Gtk.ListBox.new()
+            self.__pages_listbox.set_sort_func(self.__sort_func)
             self.__pages_listbox.show()
             self.__separator.show()
             self.__grid.attach(self.__pages_listbox, 0, 2, 2, 1)
@@ -265,7 +278,6 @@ class SitesManagerChild(Gtk.ListBoxRow):
         """
         if netloc != self.__netloc:
             self.__netloc = netloc
-            self.__set_artwork()
             self.__netloc_label.set_text(self.__netloc)
 
     def update_label(self):
@@ -290,8 +302,11 @@ class SitesManagerChild(Gtk.ListBoxRow):
                 for child in self.__pages_listbox.get_children():
                     if child.view == current:
                         self.__pages_listbox.select_row(child)
+                        self.__current_child = child
             else:
                 self.__pages_listbox.unselect_all()
+                self.__current_child = None
+            self.__pages_listbox.invalidate_sort()
 
     @property
     def empty(self):
@@ -328,18 +343,6 @@ class SitesManagerChild(Gtk.ListBoxRow):
 #######################
 # PROTECTED           #
 #######################
-    def _on_scroll_event(self, eventbox, event):
-        """
-            Switch between children
-            @param eventbox as Gtk.EventBox
-            @param event as Gdk.Event
-        """
-        if event.direction == Gdk.ScrollDirection.UP:
-            self.__window.container.pages_manager.previous()
-        elif event.direction == Gdk.ScrollDirection.DOWN:
-            self.__window.container.pages_manager.next()
-        self.__window.container.ctrl_released()
-
     def _on_button_press_event(self, eventbox, event):
         """
             Hide popover or close view
@@ -376,11 +379,26 @@ class SitesManagerChild(Gtk.ListBoxRow):
                 popover.show()
                 return True
         except Exception as e:
-            print("SitesManagerChild::_on_button_press_event:", e)
+            Logger.error("SitesManagerChild::_on_button_press_event:", e)
 
 #######################
 # PRIVATE             #
 #######################
+    def __sort_func(self, row1, row2):
+        """
+            Sort pages
+            @param row1 as PageChildRow
+            @param row2 as PageChildRow
+        """
+        # Always show current first
+        if self.__current_child in [row1, row2]:
+            return self.__current_child == row2
+        # Unshown first
+        elif not row2.view.webview.shown and row1.view.webview.shown:
+            return True
+        else:
+            return row2.view.webview.atime > row1.view.webview.atime
+
     def __update_popover_internals(self, widget):
         """
             Little hack to manage Gtk.ModelButton text
@@ -393,36 +411,30 @@ class SitesManagerChild(Gtk.ListBoxRow):
         elif hasattr(widget, "forall"):
             GLib.idle_add(widget.forall, self.__update_popover_internals)
 
-    def __set_artwork(self):
-        """
-            Set initial artwork on widget
-        """
-        artwork = El().art.get_icon_theme_artwork(remove_www(self.__netloc),
-                                                  self.__ephemeral)
-        if artwork is not None:
-            self.__image.set_from_icon_name(artwork,
-                                            Gtk.IconSize.INVALID)
-        else:
-            favicon_path = El().art.get_favicon_path(remove_www(self.__netloc))
-            if favicon_path is not None:
-                self.__image.set_from_file(favicon_path)
-            else:
-                self.__image.set_from_icon_name("applications-internet",
-                                                Gtk.IconSize.INVALID)
-
-    def __on_webview_favicon_changed(self, webview, favicon,
-                                     icon_theme_artwork):
+    def __on_webview_favicon_changed(self, webview, surface=None):
         """
             Set favicon
             @param webview as WebView
-            @param favicon as cairo.Surface
-            @param icon_theme_artwork as str
+            @param surface as cairo.Surface
         """
-        if favicon is not None:
-            self.__image.set_from_surface(favicon)
-        elif icon_theme_artwork is not None:
-            self.__image.set_from_icon_name(icon_theme_artwork,
+        if surface is not None:
+            self.__image.set_from_surface(surface)
+            return
+
+        favicon_path = App().art.get_favicon_path(webview.uri)
+        if favicon_path is not None:
+            self.__image.set_from_file(favicon_path)
+            return
+
+        artwork = App().art.get_icon_theme_artwork(webview.uri,
+                                                   webview.ephemeral)
+        if artwork is not None:
+            self.__image.set_from_icon_name(artwork,
                                             Gtk.IconSize.INVALID)
+            return
+
+        self.__image.set_from_icon_name("applications-internet",
+                                        Gtk.IconSize.INVALID)
 
     def __on_query_tooltip(self, widget, x, y, keyboard, tooltip):
         """
@@ -482,7 +494,7 @@ class SitesManagerChild(Gtk.ListBoxRow):
             @param time as int
         """
         height = self.get_allocated_height()
-        if y > height/2:
+        if y > height / 2:
             up = False
         else:
             up = True
@@ -502,7 +514,7 @@ class SitesManagerChild(Gtk.ListBoxRow):
             @param time as int
         """
         height = self.get_allocated_height()
-        if y > height/2:
+        if y > height / 2:
             self.get_style_context().add_class("drag-up")
             self.get_style_context().remove_class("drag-down")
         else:
@@ -524,3 +536,4 @@ class SitesManagerChild(Gtk.ListBoxRow):
             Update indicataor
         """
         self.__indicator_label.mark_shown(webview)
+        self.__on_webview_favicon_changed(webview)
